from pydantic import BaseModel
from typing import List, Union, Optional


class Config(BaseModel):
    platform: str = "red"
    """
    用于绑定/获取玩家数据的平台, 一般 QQ 为 red, 其他平台可能需要根据 Koishi 适配器的具体实现来修改
    """

    tsugu_api_timeout: int = 10
    '''
    请求超时时间
    '''

    tsugu_api_proxy: str = ''
    '''
    代理地址
    '''

    tsugu_api_backend_url: str = 'http://tsugubot.com:8080'
    '''
    后端地址
    默认为 Tsugu 官方后端，若有自建后端服务器可进行修改。
    '''

    tsugu_api_backend_proxy: bool = True
    '''
    是否使用后端代理
    当设置代理地址后可修改此项以决定是否使用代理。
    默认为 True，即使用后端代理。若使用代理时后端服务器无法访问，可将此项设置为 False。
    '''

    tsugu_api_userdata_backend_url: str = 'http://tsugubot.com:8080'
    '''
    用户数据后端地址
    默认为 Tsugu 官方后端，若有自建后端服务器可进行修改。
    '''

    tsugu_api_userdata_backend_proxy: bool = True
    '''
    是否使用用户数据后端代理
    当设置代理地址后可修改此项以决定是否使用代理。
    默认为 True，即使用后端代理。若使用代理时后端服务器无法访问，可将此项设置为 False。
    '''

    tsugu_api_use_easy_bg: bool = True
    '''
    是否使用简易背景，使用可在降低背景质量的前提下加快响应速度。
    默认为 True，即使用简易背景。若不使用简易背景，可将此项设置为 False。
    '''

    tsugu_api_compress: bool = True
    '''
    是否压缩返回数据，压缩可减少返回数据大小。
    默认为 True，即压缩返回数据。若不压缩返回数据，可将此项设置为 False。
    '''

    tsugu_prefix: List[str] = ['/', '']
    '''
    命令前缀
    最后一个参数如果不是空字符串，则只有在命令前缀符合时才会触发命令。
    '''

    tsugu_allow_gap_less: bool = True
    '''
    是否允许命令与参数之间没有空格
    '''

    tsugu_get_remote_user_data_max_retry: int = 3
    '''
    获取远程用户数据最大重试次数
    '''

    tsugu_token_name: str = "Tsugu"
    '''
    bandori station token
    '''
    tsugu_bandori_station_token: str = "ZtV4EX2K9Onb"
    '''
    bandori station token
    '''

    tsugu_ban_gacha_simulate_group_data: List = []
    '''
    需要关闭模拟抽卡的群
    '''



import nonebot
import tsugu_async.handler
from nonebot import require
from nonebot import on_message
from nonebot.log import logger
from nonebot.params import Depends
from nonebot.adapters import Event, Bot
from tsugu_async.config import config as tsugu_config
from tsugu_api_async import settings as tsugu_api_config
from nonebot.plugin import PluginMetadata, inherit_supported_adapters

require("nonebot_plugin_alconna")
require("nonebot_plugin_userinfo")

from nonebot_plugin_userinfo import get_user_info
from nonebot_plugin_alconna import UniMessage, Target, Image, Text, At

from .config import Config


if hasattr(nonebot, "get_plugin_config"):
    plugin_config = nonebot.get_plugin_config(Config)
else:
    plugin_config = Config.parse_obj(nonebot.get_driver().config)

if hasattr(plugin_config, "model_dump"):
    plugin_config_dict = plugin_config.model_dump()
else:
    plugin_config_dict = plugin_config.dict()


__plugin_meta__ = PluginMetadata(
    name="BanG Dream! Tsugu Frontend",
    description="基于 tsugu-python-frontend 的 tsugu-bangdream-bot 插件",
    usage="\n".join([f"{k}:\n{v}" for k, v in tsugu_config._help_doc_dict.items()]),
    type="application",
    homepage="https://github.com/zhaomaoniu/nonebot-plugin-tsugu-frontend",
    config=Config,
    supported_adapters=inherit_supported_adapters(
        "nonebot_plugin_alconna", "nonebot_plugin_userinfo"
    ),
)


# 更新配置对象
for key, value in plugin_config_dict.items():
    if key.startswith("tsugu_api_"):
        attr_name = key.replace("tsugu_api_", "")
        if hasattr(tsugu_api_config, attr_name):
            setattr(tsugu_api_config, attr_name, value)
    elif key.startswith("tsugu_"):
        attr_name = key.replace("tsugu_", "")
        if hasattr(tsugu_config, attr_name):
            setattr(tsugu_config, attr_name, value)


async def get_target(event: Event, bot: Bot):
    return UniMessage.get_target(event, bot)


@on_message(priority=10, block=False).handle()
async def tsugu_handle(
    bot: Bot,
    event: Event,
    target: Target = Depends(get_target),
):
    user_info = await get_user_info(bot, event, event.get_user_id())

    if not user_info:
        logger.warning("Failed to get user info")
        return None

    result = await tsugu_async.handler(
        message=event.get_message().extract_plain_text(),
        user_id=user_info.user_id,
        platform=plugin_config.platform,
        channel_id=(
            target.id if (target.channel or not target.private) else user_info.user_id
        ),
    )

    if not result:
        return None

    message = UniMessage([At("user", user_info.user_id), Text(" ")])

    for content in result:
        if isinstance(content, str):
            message.append(Text(content))
        elif isinstance(content, bytes):
            message.append(Image(raw=content))

    await message.send(target, bot)